unit UReadOnlyByDefault;

interface

uses
  SysUtils, ToolsAPI;

procedure Register;

implementation

var
  FNotifierIndex: Integer;
  FSourcePath: string;

procedure SetAsReadOnly(Module: IOTAModule);
var
  editBuffer: IOTAEditBuffer;
begin
  if Module = nil then
    Exit;
  if Pos(FSourcePath, LowerCase(Module.FileName)) <> 1 then
    Exit;
  if not Supports(Module.CurrentEditor, IOTAEditBuffer, editBuffer) then
    Exit;
  editBuffer.IsReadOnly := True;
end;

type
  TIDENotifier = class(TNotifierObject, IOTAIDENotifier)
  private
    procedure FileNotification(NotifyCode: TOTAFileNotification;
      const FileName: string; var Cancel: Boolean);
    procedure BeforeCompile(const Project: IOTAProject; var Cancel: Boolean); overload;
    procedure AfterCompile(Succeeded: Boolean); overload;
  end;

{ TIDENotifier }

procedure TIDENotifier.FileNotification(NotifyCode: TOTAFileNotification;
  const FileName: string; var Cancel: Boolean);
begin
  if NotifyCode = ofnFileOpened then
  begin
    SetAsReadOnly((BorlandIDEServices as IOTAModuleServices).FindModule(FileName));
  end;
end;

procedure TIDENotifier.AfterCompile(Succeeded: Boolean);
begin
  { Do nothing }
end;

procedure TIDENotifier.BeforeCompile(const Project: IOTAProject;
  var Cancel: Boolean);
begin
  { Do nothing }
end;

procedure Register;
var
  moduleServices: IOTAModuleServices;
  i: Integer;
begin
  FSourcePath := LowerCase(ExtractFilePath(ExtractFileDir(ParamStr(0))) + 'source\');
  FNotifierIndex := (BorlandIDEServices as IOTAServices).AddNotifier(TIDENotifier.Create);

  moduleServices := (BorlandIDEServices as IOTAModuleServices);
  for i := 0 to moduleServices.ModuleCount-1 do
  begin
    SetAsReadOnly(moduleServices.Modules[i]);
  end;
end;

procedure Unregister;
begin
  if FNotifierIndex >= 0 then
    (BorlandIDEServices as IOTAServices).RemoveNotifier(FNotifierIndex);
end;

initialization
finalization
  Unregister;
end.
